/*******************************************************************************
 * Copyright (c) 2005, 2010 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: ExtensibleEObjectTableSection.java,v 1.5 2010-03-29 13:52:37 paules Exp $
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.hyades.test.ui.editor.form.util;

import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.hyades.test.ui.adapter.TestWorkbenchAdapter;
import org.eclipse.hyades.test.ui.internal.editor.form.base.FormWidgetFactory;
import org.eclipse.hyades.test.ui.internal.editor.form.base.StructuredViewerPart;
import org.eclipse.hyades.test.ui.internal.editor.form.util.StructuredViewerSection;
import org.eclipse.hyades.ui.internal.provider.WorkbenchAdapterLabelProvider;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IBaseLabelProvider;
import org.eclipse.jface.viewers.IContentProvider;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredViewer;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.ControlListener;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.actions.SelectionListenerAction;

/**
 * <p>An <code>EObject</code> table view section with configurable buttons.</p>
 * 
 * <p>The sole constructor of this class takes all the button actions in an array, which
 * will be used to create the buttons and will be invoked when the button is selected.</p>
 * 
 * 
 * @author  Bianca Jiang Xue
 * @author  Paul Slauenwhite
 * @version March 29, 2010
 * @since   February 1, 2005
 */
public class ExtensibleEObjectTableSection extends StructuredViewerSection implements ISelectionChangedListener, IDoubleClickListener{

	private int tableStyle = SWT.MULTI | SWT.FULL_SELECTION;
	private int numColumns;
	private IAction[] buttonActions;
	private EditorForm editorForm;
	private EStructuralFeature eStructuralFeature;
	private IContentProvider tableContentProvider;
	private ITableLabelProvider tableLabelProvider;

	/**
	 * Constructor to created an extensible object table section.
	 * 
	 * @param editorForm The editor form the form of the extensible object table section.
	 * @param eStructuralFeature The {@link EStructuralFeature} of the extensible object table section.
	 * @param numOfColumns The number of columns of extensible object table section.
	 * @param theButtonActions The actions used to created the buttons provided by the extensible object table section.
	 */
	public ExtensibleEObjectTableSection(EditorForm editorForm, EStructuralFeature eStructuralFeature, int numOfColumns, IAction[] theButtonActions){
		
		super(editorForm);
		
		this.editorForm = editorForm;
		this.eStructuralFeature = eStructuralFeature;
		this.numColumns = numOfColumns;
		this.buttonActions = theButtonActions;
		String[] buttonLabels = new String[buttonActions.length];
		for(int i = 0; i < buttonActions.length; i++)
		{
			buttonLabels[i] = buttonActions[i].getText();
		}
		
		super.setButtonLabels(buttonLabels);
	}
	
	/**
	 * @see org.eclipse.update.ui.forms.internal.FormSection#dispose()
	 */
	public void dispose()
	{
		eStructuralFeature = null;
		
		for(int i = 0; i < buttonActions.length; i++)
		{
			if(buttonActions[i] != null)
				buttonActions[i] = null;
		}

		super.dispose();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.hyades.test.ui.internal.editor.form.util.StructuredViewerSection#createViewerPart(java.lang.String[])
	 */
	protected StructuredViewerPart createViewerPart(String[] buttonLabels) 
	{
		StructuredViewerPart tablePart = new StructuredViewerPart(buttonLabels)
		{
			/*
			 * @see StructuredViewerPart#createStructuredViewer(Composite, FormWidgetFactory)
			 */
			protected StructuredViewer createStructuredViewer(Composite parent, int style, FormWidgetFactory factory)
			{				
				// create a table with numColumns columns
				Table table = new Table(parent, style);
				table.setLinesVisible(true);
				table.setHeaderVisible(false);
				for(int i = 0; i < numColumns; i++)
				{
					TableColumn column = new TableColumn(table, SWT.NONE);
					column.setResizable(true);
				}
				table.addControlListener( new ControlListener() {
					public void controlMoved(ControlEvent e) {}

					public void controlResized(ControlEvent e) 
					{
						if(e.getSource() instanceof Table)
						{
							Table table = ((Table)e.getSource());
							int width = table.getSize().x;
							int gridLineWidth = table.getGridLineWidth();
							TableColumn[] columns = table.getColumns();
							if( columns.length == 3 )
							{
								columns[0].setWidth(width*2/5);
								columns[1].setWidth(width/5);
								columns[2].setWidth(width*2/5 + gridLineWidth);
							}
							else
							{
								for(int i = 0; i < columns.length; i++)
									columns[i].setWidth(width/columns.length);
							}
						}

					}
				});
			
				parent.pack();
				
				TableViewer tableViewer = new TableViewer(table);
				tableViewer.addSelectionChangedListener(ExtensibleEObjectTableSection.this);
				tableViewer.addDoubleClickListener(ExtensibleEObjectTableSection.this);
				return tableViewer;
			}
			public void buttonSelected(Button button, int index)
			{
				ExtensibleEObjectTableSection.this.buttonSelected(index);
				if(isHandlingDefaultButton())
					button.getShell().setDefaultButton(null);
			}			
		};
		
		return tablePart;
	}
	
	protected void buttonSelected(int index)
	{
		if(!isReadOnly())
		{
			if( index >=0 && index < buttonActions.length )
				buttonActions[index].run();
		}
	
		updateActionsAndButtons(getStructuredSelection());
	}

	/* (non-Javadoc)
	 * @see org.eclipse.hyades.test.ui.internal.editor.form.base.FormSection#createClient(org.eclipse.swt.widgets.Composite, org.eclipse.hyades.test.ui.internal.editor.form.base.FormWidgetFactory)
	 */
	public Composite createClient(Composite parent, FormWidgetFactory factory) 
	{
		setFormWidgetFactory(factory);
		Composite container = createClientContainer(parent, 2, getFormWidgetFactory());
		createViewerPartControl(container, getTableStyle(), 2, getFormWidgetFactory());
		getTableViewer().getControl().setFocus();

		adjustClient(parent);

		getFormWidgetFactory().paintBordersFor(container);
		return container;
	}
	
	/**
	 * @see org.eclipse.hyades.test.ui.internal.editor.form.base.BaseTreeSection#adjustClient(org.eclipse.swt.widgets.Composite)
	 */
	protected void adjustClient(Composite parent)
	{
		getTableViewer().setContentProvider(getContentProvider());
		getTableViewer().setLabelProvider(getLabelProvider());

		for(int i = 0; i < buttonActions.length; i++)
		{
			if(buttonActions[i] != null)
			{
				getTablePart().setButtonEnabled(i, buttonActions[i].isEnabled());
			}
				
		}
	}
	
	/**
	 * @see org.eclipse.hyades.test.ui.internal.editor.form.base.BaseStructuredViewerSection#updateActionsAndButtons(org.eclipse.jface.viewers.IStructuredSelection)
	 */
	protected void updateActionsAndButtons(IStructuredSelection structuredSelection)
	{
		if(buttonActions == null || buttonActions.length < 1)
			return;
			
		for(int i = 0; i < buttonActions.length; i++)
		{
			if(buttonActions[i] != null)
			{
				if(buttonActions[i] instanceof SelectionListenerAction)
					((SelectionListenerAction)buttonActions[i]).selectionChanged(structuredSelection);
				getTablePart().setButtonEnabled(i, buttonActions[i].isEnabled());
			}
		}
	}

	/**
	 * Sets the content provider this table section should use.
	 * 
	 * @param contentProvider
	 */
	public void setContentProvider(IContentProvider contentProvider)
	{
		this.tableContentProvider = contentProvider;
	}
	
	/**
	 * Returns this section's content provider. The default provider
	 * {@link EObjectTableContentProvider} will be returned if there is not one
	 * already set by {@link #setContentProvider()}.
	 *
	 * @return IContentProvider
	 */
	protected IContentProvider getContentProvider(){
		
		if(tableContentProvider == null){
			tableContentProvider = new EObjectTableContentProvider(editorForm, eStructuralFeature);
		}
		
		return this.tableContentProvider;
	}
	
	/**
	 * Returns the feature this table section displays/edits.
	 * @return
	 */
	protected EStructuralFeature getEStructuralFeature()
	{
		return eStructuralFeature;
	}
	
	/**
	 * Returns this section's label provider.
	 *
	 * @return ILabelProvider
	 * @see	#getLabelProvider()
	 */
	public IBaseLabelProvider getLabelProvider()
	{
		if(tableLabelProvider == null)
			return new WorkbenchAdapterLabelProvider(TestWorkbenchAdapter.class);
		else
			return tableLabelProvider;
	}
	
	/**
	 * Sets the label provider this table section should use.
	 * @param provider
	 */
	public void setLabelProvider(ITableLabelProvider provider)
	{
		this.tableLabelProvider = provider;
	}
	
	/**
	 * Sets the table style.  The default value is 
	 * <code>SWT.FULL_SELECTION | SWT.MULTI</code>
	 * @param tableStryle
	 */
	public void setTableStyle(int tableStyle)
	{
		this.tableStyle = tableStyle;
	}	
	
	/**
	 * Gets the table style.  The default value is 
	 * <code>SWT.FULL_SELECTION | SWT.MULTI</code>
	 * @return int
	 */
	public int getTableStyle()
	{
		return tableStyle;
	}
	
	/**
	 * Returns this section's table part.
	 * @return TablePart
	 */
	public StructuredViewerPart getTablePart()
	{
		return (StructuredViewerPart)getViewerPart();
	}

	/**
	 * Returns this section's table viewer.
	 * @return TableViewer
	 */
	public TableViewer getTableViewer()
	{
		return (TableViewer)getTablePart().getViewer();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ISelectionChangedListener#selectionChanged(org.eclipse.jface.viewers.SelectionChangedEvent)
	 */
	public void selectionChanged(SelectionChangedEvent event) 
	{
		selectionChanged((IStructuredSelection)event.getSelection());
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IDoubleClickListener#doubleClick(org.eclipse.jface.viewers.DoubleClickEvent)
	 */
	public void doubleClick(DoubleClickEvent event){
		
		//Only process this mouse event if the editor is not opened in read-only mode:
		if(!isReadOnly()){
			handleDoubleClick((IStructuredSelection)event.getSelection());
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object, int)
	 */
	public String getColumnText(Object element, int columnIndex) 
	{
		return element.toString();
	}

}
