/*******************************************************************************
 * Copyright (c) 2011 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: 
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

package org.eclipse.hyades.trace.ui.internal.export;

import java.io.DataOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;

abstract class AbstractBinaryMessage {

	// 0 - 1000 System messages
	final int BF_ENCODING_ID = 1;
	final int BF_FREQ_ID = 2;

	// 1000 - 32767 Data messages
	final int BF_NODE_ID = 1001;
	final int BF_PROCESS_CREATE_ID = 1002;
	final int BF_AGENT_CREATE_ID = 1003;
	final int BF_AGENT_DESTROY_ID = 1004;
	final int BF_TRACE_START_ID = 1005;
	final int BF_TRACE_END_ID = 1006;
	final int BF_FILTER_ID = 1007;
	final int BF_OPTION_ID = 1008;
	final int BF_THREAD_START_ID = 1009;
	final int BF_THREAD_END_ID = 1010;
	final int BF_CLASS_DEF_ID = 1011;
	final int BF_METHOD_DEF_ID = 1012;
	final int BF_OBJ_ALLOC_ID = 1013;
	final int BF_METHOD_ENTRY_ID = 1014;
	final int BF_METHOD_EXIT_ID = 1015;
	final int BF_METHOD_CALL_ID = 1016;
	final int BF_METHOD_RETURN_ID = 1017;
	final int BF_INVOCATION_CONTEXT_ID = 1018;
	final int BF_OBJ_DEF_ID = 1019;
	final int BF_VALUE_ID = 1020;
	final int BF_METHOD_COUNT_ID = 1021;
	final int BF_LINE_ID = 1022;
	final int BF_GC_START_ID = 1023;
	final int BF_OBJ_FREE_ID = 1024;
	final int BF_CLASS_UNLOAD_ID = 1025;
	final int BF_OBJ_MOVE_ID = 1026;
	final int BF_GC_FINISH_ID = 1027;
	final int BF_THROW_ID = 1028;
	final int BF_CATCH_ID = 1029;
	final int BF_RUNTIME_INIT_DONE_ID = 1030;
	final int BF_RUNTIME_SHUTDOWN_ID = 1031;
	final int BF_MON_WAIT_ID = 1032;
	final int BF_MON_WAITED_ID = 1033;
	final int BF_MON_CONTENDED_ENTER_ID = 1034;
	final int BF_MON_CONTENDED_ENTERED_ID = 1035;

	// Messages; which are not listed in public DTD
	final int BF_AG_METHOD_ENTRY_ID = 1036;
	final int BF_AG_METHOD_EXIT_ID = 1037;
	final int BF_HEAP_DUMP_DEF_ID = 1038;
	final int BF_GC_ROOT_ID = 1039;
	final int BF_OBJ_REF_ID = 1040;
	final int BF_CUSTOM_ID = 1041;
	final int BF_CALL_ID_NOTIFY_ID = 1042;
	final int BF_CALL_ID_INTERRUPT_ID = 1043;
	final int BF_CALL_ID_START_ID = 1044;
	final int BF_CALL_ID_SLEEP_START_ID = 1045;
	final int BF_CALL_ID_SLEEP_END_ID = 1046;

	public abstract int getSize();

	public abstract short getId();

	public void write(OutputStream os) throws IOException {
		writeShort(os, getId()); // id
		writeInt(os, getSize()); // id
	}

	// -------------------

	void writeByte(OutputStream os, byte b) throws IOException {
		os.write(b);
	}

	void writeShort(OutputStream os, short s) throws IOException {
		DataOutputStream dos = new DataOutputStream(os);
		dos.writeShort(s);
	}

	void writeInt(OutputStream os, int i) throws IOException {
		DataOutputStream dos = new DataOutputStream(os);
		dos.writeInt(i);
	}

	void writeLong(OutputStream os, long l) throws IOException {
		DataOutputStream dos = new DataOutputStream(os);
		dos.writeLong(l);
	}

	void writeDouble(OutputStream os, double d) throws IOException {
		DataOutputStream dos = new DataOutputStream(os);
		dos.writeDouble(d);
	}

	void writeString(OutputStream os, String s) throws IOException {
		DataOutputStream dos = new DataOutputStream(os);
		if (s == null) {
			dos.writeByte(0);
			return;
		}
		
		byte[] b = s.getBytes("UTF-8");
		dos.write(b);
		
		dos.writeByte(0);
	}

	void writeStringArr(OutputStream os, String[] sarr) throws IOException {
		if (sarr == null) {
			DataOutputStream dos = new DataOutputStream(os);
			dos.writeByte(0);
			return;
		}

		for (int x = 0; x < sarr.length; x++) {
			writeString(os, sarr[x]);
		}
	}

	void writeLongArr(OutputStream os, long[] larr) throws IOException {

		if (larr == null) {
			DataOutputStream dos = new DataOutputStream(os);
			dos.writeByte(0);
			return;
		}

		for (int x = 0; x < larr.length; x++) {
			writeLong(os, larr[x]);
		}
	}

	int getByteSize() {
		return 1;
	}

	int getShortSize() {
		return 2;
	}

	int getIntSize() {
		return 4;
	}

	int getLongSize() {
		return 8;
	}

	int getDoubleSize() {
		return 8;
	}

	int getStringSize(String s) {
		if(s == null) return 1;
		
		byte[] strarr = null;
		try {
			strarr = s.getBytes("UTF-8");
			return strarr.length + 1;
		} catch(UnsupportedEncodingException e) { /* this shouldn't happen for utf */ return 1;}
		
	}

	int getStringArrSize(String[] sarr) {
		if (sarr == null || sarr.length == 0) {
			return 1;
		}

		int size = 0;
		for (int x = 0; x < sarr.length; x++) {

			if (sarr[x] != null) {
				size += sarr[x].length() + 1;
			} else {
				size += 1;
			}
		}
		return size;
	}

	int getSize(long larr[]) {
		if (larr == null || larr.length == 0) {
			return 0;
		}
		return 8 * larr.length;
	}

	// ---------------------------------------

}

class BFormatHeader extends AbstractBinaryMessage {

	@Override
	public int getSize() {
		return 0;
	}

	@Override
	public short getId() {
		return 0;
	}
	
	@Override
	public void write(OutputStream os) throws IOException {
		// don't call the super writer

		writeByte(os, (byte)0); // buffer = write(buffer, indent);
		writeByte(os, (byte)'T');
		writeByte(os, (byte)'B');
		writeByte(os, (byte)'F');
		writeByte(os, (byte)1); // write(buffer, version_major);
		writeByte(os, (byte)0); // write(buffer, version_minor);

		writeByte(os, (byte)1); // platform
		writeByte(os, (byte)1); // endianness

		writeInt(os, 0); // data offset
		
	}
	
}


class BEncodingMessage extends AbstractBinaryMessage {
	String _format = "UTF-8"; //$NON-NLS-1$
			
	public void write(OutputStream o) throws IOException {
		super.write(o);
		writeString(o, _format);

	}

	@Override
	public int getSize() {
		return getStringSize(_format);
	}

	@Override
	public short getId() {
		return BF_ENCODING_ID;
	}
	
}

class BFrequencyMessage extends AbstractBinaryMessage {

	@Override
	public int getSize() {
		return getLongSize();
	}

	@Override
	public short getId() {
		return BF_FREQ_ID;
	}
	
	@Override
	public void write(OutputStream o) throws IOException {
		super.write(o);
		writeLong(o, 0);
	}
	
}

class CNodeMessage extends AbstractBinaryMessage {

	String _node_id = null;
	String _hostname = null;
	String _ip = null;
	int _timezone = 0;
	long _timestamp = 0;
	
	@Override
	public int getSize() {
		return (getStringSize(_node_id) +
				getStringSize(_hostname) +
				getStringSize(_ip) +
				getIntSize() + // _timezone
				getLongSize()); // _timestamp
	}
	
	@Override
	public short getId() {
		return BF_NODE_ID;
	}
	
	@Override
	public void write(OutputStream o) throws IOException {
		super.write(o);
		
		writeString(o, _node_id);
		writeString(o, _hostname);
		writeString(o, _ip);
		writeInt(o, _timezone);
		writeLong(o, _timestamp);
		
	}

}


class CProcessCreateMessage extends AbstractBinaryMessage {
	String _process_id = null;
	String _name = null;
	int _pid = 0;
	String _node_id_ref = null;
	long _timestamp = 0;
	String _exe = null;

	@Override
	public int getSize() {
		return (getStringSize(_process_id) +
				getStringSize(_name) +
				getIntSize() + // pid
				getStringSize(_node_id_ref) +
				getLongSize() + // timestamp
				getStringSize(_exe));
	}

	@Override
	public short getId() {
		return BF_PROCESS_CREATE_ID;
	}
	
	@Override
	public void write(OutputStream os) throws IOException {
		super.write(os);

		writeString(os, _process_id);
		writeString(os, _name);
		writeInt(os, _pid);
		
		writeString(os, _node_id_ref);
		writeLong(os, _timestamp);
		writeString(os, _exe);
		
	}

}




class CAgentCreateMessage extends AbstractBinaryMessage {
	String _agent_id = null;
	String _process_id_ref = null;
	String _name = null;
	String _type = null;
	long _timestamp = 0;
	String _params = null;
	String _version = null;

	@Override
	public int getSize() {
		return (getStringSize(_agent_id) +
				getStringSize(_process_id_ref) +
				getStringSize(_name) +
				getStringSize(_type) +
				getLongSize() + // _timestamp
				getStringSize(_params) +
				getStringSize(_version));	}

	@Override
	public short getId() {
		return BF_AGENT_CREATE_ID;
	}
	
	@Override
	public void write(OutputStream os) throws IOException {
		super.write(os);

		writeString(os, _agent_id);
		writeString(os, _process_id_ref);
		writeString(os, _name);
		writeString(os, _type);
		writeLong(os, _timestamp);
		writeString(os, _params);
		writeString(os, _version);
				
	}
	
}




class CAgentDestroyMessage extends AbstractBinaryMessage {
	String _agent_id_ref = null;
	long _timestamp = 0;

	@Override
	public int getSize() {
		return (getStringSize(_agent_id_ref) +
				getLongSize()); // _timestamp
	}

	@Override
	public short getId() {
		return BF_AGENT_DESTROY_ID;
	}
	
	@Override
	public void write(OutputStream os) throws IOException {
		super.write(os);
		writeString(os, _agent_id_ref);
		writeLong(os, _timestamp);
	}
	
}




class CTraceStartMessage extends AbstractBinaryMessage {
	String _trace_id;
	String _agent_id_ref;
	long _timestamp;
	String _col_value;

	@Override
	public int getSize() {
		return (getStringSize(_trace_id) +
				getStringSize(_agent_id_ref) +
				getLongSize() + // timestamp
				getStringSize(_col_value));
	}

	@Override
	public short getId() {
		return BF_TRACE_START_ID;
	}
	
	@Override
	public void write(OutputStream os) throws IOException {
		super.write(os);
		
		writeString(os, _trace_id);
		writeString(os, _agent_id_ref);
		writeLong(os, _timestamp);
		writeString(os, _col_value);
	}
	
}



class CTraceEndMessage extends AbstractBinaryMessage {
	String _trace_id_ref;
	long _timestamp;
	String _col_value;

	@Override
	public int getSize() {
		return (getStringSize(_trace_id_ref) +
				getLongSize() + // timestamp
				getStringSize(_col_value));
	}

	@Override
	public short getId() {
		return BF_TRACE_END_ID;
	}
	
	@Override
	public void write(OutputStream os) throws IOException {
		super.write(os);
		
		writeString(os, _trace_id_ref);
		writeLong(os, _timestamp);
		writeString(os, _col_value);
	}
	
}


class CFilterMessage extends AbstractBinaryMessage {
	String _trace_id_ref;
	String _pattern;
	String _gen_pattern;
	String _mode;
	String _meth_pattern;
	String _meth_gen_pattern;
	String _meth_mode;

	@Override
	public int getSize() {
		return (getStringSize(_trace_id_ref) +
				getStringSize(_pattern) +
				getStringSize(_gen_pattern) +
				getStringSize(_mode) +
				getStringSize(_meth_pattern) +
				getStringSize(_meth_gen_pattern) +
				getStringSize(_meth_mode));
	}

	@Override
	public short getId() {
		return BF_FILTER_ID;
	}
	
	@Override
	public void write(OutputStream os) throws IOException {
		super.write(os);
		
		writeString(os, _trace_id_ref);
		writeString(os, _pattern);
		writeString(os, _gen_pattern);
		writeString(os, _mode);
		writeString(os, _meth_pattern);
		writeString(os, _meth_gen_pattern);
		writeString(os, _meth_mode);		
	}
	
}


class COptionMessage extends AbstractBinaryMessage {
	String _trace_id_ref;
	String _key;
	String _value;

	@Override
	public int getSize() {
		return (getStringSize(_trace_id_ref) +
				getStringSize(_key) +
				getStringSize(_value));
	}

	@Override
	public short getId() {
		return BF_OPTION_ID;
	}
	
	@Override
	public void write(OutputStream os) throws IOException {
		super.write(os);

		writeString(os, _trace_id_ref);
		writeString(os, _key);
		writeString(os, _value);
				
	}
	
}



class CThreadStartMessage extends AbstractBinaryMessage {
	long _trans_thread_id;
	long _thread_id;
	long _timestamp;
	String _group_name;
	String _parent_group_name;
	long _trans_obj_id_ref;
	long _obj_id_ref;
	String _thread_name;
	String _col_value;
	String _trace_id_ref;

	@Override
	public int getSize() {
		return (getLongSize() + // trans_thread_id
				getLongSize() + // thread_id
				getLongSize() + // timestamp
				getStringSize(_group_name) +
				getStringSize(_parent_group_name) +
				getLongSize() + // trans_obj_id_ref
				getLongSize() + // obj_id_ref
				getStringSize(_thread_name) +
				getStringSize(_col_value) +
				getStringSize(_trace_id_ref));

	}

	@Override
	public short getId() {
		return BF_THREAD_START_ID;
	}
	
	@Override
	public void write(OutputStream os) throws IOException {
		super.write(os);
		
		writeLong(os, _trans_thread_id);
		writeLong(os, _thread_id);
		writeLong(os, _timestamp);
		
		writeString(os, _group_name);
		writeString(os, _parent_group_name);
		
		writeLong(os, _trans_obj_id_ref);
		writeLong(os, _obj_id_ref);
		
		writeString(os, _thread_name);
		writeString(os, _col_value);
		writeString(os, _trace_id_ref);
		
	}
	
}


class CThreadEndMessage extends AbstractBinaryMessage {
	long _trans_thread_id_ref;
	long _thread_id_ref;
	long _timestamp;
	String _col_value;
	String _trace_id_ref;

	
	@Override
	public int getSize() {
		return (getLongSize() + // trans_thread_id_ref
				getLongSize() + // thread_id_ref
				getLongSize() + // timestamp
				getStringSize(_col_value) +
				getStringSize(_trace_id_ref));
	}

	@Override
	public short getId() {
		return BF_THREAD_END_ID;
	}
	
	@Override
	public void write(OutputStream os) throws IOException {
		super.write(os);

		writeLong(os, _trans_thread_id_ref);
		writeLong(os, _thread_id_ref);
		writeLong(os, _timestamp);
		writeString(os, _col_value);
		writeString(os, _trace_id_ref);
		
	}
	
}


class CClassDefMessage extends AbstractBinaryMessage {
	long _trans_thread_id_ref;
	long _thread_id_ref;
	long _timestamp;
	int _num_interfaces;
	String _interface_names;
	long _trans_class_id;
	long _class_id;
	String _source_name;
	String _class_loader;
	String _superclass;
	long _trans_obj_id_ref;
	long _obj_id_ref;
	String _name;
	String _access;
	int _num_static_fields;
	int _num_methods;
	int _num_instance_fields;
	String _col_value;
	String _trace_id_ref;

	@Override
	public int getSize() {
		return (getLongSize() + // _trans_thread_id_ref)
				getLongSize() + // _thread_id_ref
				getLongSize() + // _timestamp
				getIntSize() + // _num_interfaces
				getStringSize(_interface_names) +
				getLongSize() + // _trans_class_id
				getLongSize() + // _class_id
				getStringSize(_source_name) +
				getStringSize(_class_loader) +
				getStringSize(_superclass) +
				getLongSize() + // _trans_obj_id_ref
				getLongSize() + // _obj_id_ref
				getStringSize(_name) +
				getStringSize(_access) +
				getIntSize() + // _num_static_fields
				getIntSize() + // _num_methods
				getIntSize() + // _num_instance_fields
				getStringSize(_col_value) +
				getStringSize(_trace_id_ref));
	}

	@Override
	public short getId() {
		return BF_CLASS_DEF_ID;
	}
	
	@Override
	public void write(OutputStream os) throws IOException {
		super.write(os);

		writeLong(os, _trans_thread_id_ref);
		writeLong(os, _thread_id_ref);
		writeLong(os, _timestamp);
		
		writeInt(os, _num_interfaces);
		writeString(os, _interface_names);
		
		writeLong(os, _trans_class_id);
		writeLong(os, _class_id);
		writeString(os, _source_name);
		writeString(os, _class_loader);
		writeString(os, _superclass);
		
		writeLong(os, _trans_obj_id_ref);
		writeLong(os, _obj_id_ref);
		writeString(os, _name);
		
		writeString(os, _access);
		writeInt(os, _num_static_fields);
		writeInt(os, _num_methods);
		
		writeInt(os, _num_instance_fields);
		writeString(os, _col_value);
		writeString(os, _trace_id_ref);

	}
	
}


class CMethodDefMessage extends AbstractBinaryMessage {
	String _name;
	String _sig;
	String _visibility;
	byte _is_native;
	byte _is_abstract;
	byte _is_static;
	byte _is_sync;
	String _exceptions;
	long _start_line;
	long _end_line;
	String _sig_notation;
	long _trans_class_id_ref;
	long _class_id_ref;
	long _method_id;
	String _col_value;
	String _trace_id_ref;

	@Override
	public int getSize() {
		return (getStringSize(_name) +
				getStringSize(_sig) +
				getStringSize(_visibility) +
				getByteSize() + // is_native
				getByteSize() + // is_abstract
				getByteSize() + // is_static
				getByteSize() + // is_sync
				getStringSize(_exceptions) +
				getLongSize() + // start_line
				getLongSize() + // end_line
				getStringSize(_sig_notation) +
				getLongSize() + // trans_class_id_ref
				getLongSize() + // class_id_ref
				getLongSize() + // method_id
				getStringSize(_col_value) +
				getStringSize(_trace_id_ref));

	}

	@Override
	public short getId() {
		return BF_METHOD_DEF_ID;
	}
	
	@Override
	public void write(OutputStream os) throws IOException {
		super.write(os);

		writeString(os, _name);
		writeString(os, _sig);
		writeString(os, _visibility);
		writeByte(os, _is_native);
		writeByte(os, _is_abstract);
		writeByte(os, _is_static);
		writeByte(os, _is_sync);
		writeString(os, _exceptions);
		writeLong(os, _start_line);
		writeLong(os, _end_line);
		writeString(os, _sig_notation);
		writeLong(os, _trans_class_id_ref);
		writeLong(os, _class_id_ref);
		writeLong(os, _method_id);
		writeString(os, _col_value);
		writeString(os, _trace_id_ref);
	}
	
}


class CObjAllocMessage extends AbstractBinaryMessage {
	long _trans_thread_id_ref;
	long _thread_id_ref;
	long _timestamp;
	byte _is_array;
	long _trans_obj_id_ref;
	long _obj_id_ref;
	long _size;
	long _line;
	long _method_id;
	long _trans_class_id_ref;
	long _class_id_ref;
	String _context_data;
	String _col_value;
	String _trace_id_ref;

	@Override
	public int getSize() {
		return (getLongSize() + // _trans_thread_id_ref
				getLongSize() + // _thread_id_ref
				getLongSize() + // _timestamp
				getByteSize() + // _is_array
				getLongSize() + // _trans_obj_id_ref
				getLongSize() + // _obj_id_ref
				getLongSize() + // _size
				getLongSize() + // _line
				getLongSize() + // _method_id
				getLongSize() + // _trans_class_id_ref
				getLongSize() + // _class_id_ref
				getStringSize(_context_data) +
				getStringSize(_col_value) +
				getStringSize(_trace_id_ref));
	}

	@Override
	public short getId() {
		return BF_OBJ_ALLOC_ID;
	}
	
	@Override
	public void write(OutputStream os) throws IOException {
		super.write(os);
		writeLong(os, _trans_thread_id_ref);
		writeLong(os, _thread_id_ref);
		writeLong(os, _timestamp);
		writeByte(os, _is_array);
		writeLong(os, _trans_obj_id_ref);
		writeLong(os, _obj_id_ref);
		writeLong(os, _size);
		writeLong(os, _line);
		writeLong(os, _method_id);
		writeLong(os, _trans_class_id_ref);
		writeLong(os, _class_id_ref);
		writeString(os, _context_data);
		writeString(os, _col_value);
		writeString(os, _trace_id_ref);		
	}
	
}



class CMethodEntryMessage extends AbstractBinaryMessage {
	long _trans_thread_id_ref;
	long _thread_id_ref;
	long _timestamp;
	long _method_id_ref;
	int _ticket;
	long _trans_obj_id_ref;
	long _class_id_ref;
	long _thread_cpu_time;
	long _seq_counter;
	long _stack_depth;
	String _col_value;
	String _trace_id_ref;

	@Override
	public int getSize() {
		return (getLongSize() + // trans_thread_id_ref
				getLongSize() + // thread_id_ref
				getLongSize() + // timestamp
				getLongSize() + // method_id_ref
				getIntSize() + // ticket
				getLongSize() + // trans_obj_id_ref
				getLongSize() + // class_id_ref
				getLongSize() + // thread_cpu_time
				getLongSize() + // seq_counter
				getLongSize() + // stack_depth
				getStringSize(_col_value) +
				getStringSize(_trace_id_ref));
	}

	@Override
	public short getId() {
		return BF_METHOD_ENTRY_ID;
	}
	
	@Override
	public void write(OutputStream os) throws IOException {
		super.write(os);
		writeLong(os, _trans_thread_id_ref);
		writeLong(os, _thread_id_ref);
		writeLong(os, _timestamp);
		writeLong(os, _method_id_ref);
		writeInt(os, _ticket);
		writeLong(os, _trans_obj_id_ref);
		writeLong(os, _class_id_ref);
		writeLong(os, _thread_cpu_time);
		writeLong(os, _seq_counter);
		writeLong(os, _stack_depth);
		writeString(os, _col_value);
		writeString(os, _trace_id_ref);
		
	}
	
}



class CMethodExitMessage extends AbstractBinaryMessage {
	long _trans_thread_id_ref;
	long _thread_id_ref;
	long _timestamp;
	int _ticket;
	long _thread_cpu_time;
	long _method_id_ref;
	long _trans_obj_id_ref;
	long _obj_id_ref;
	long _trans_class_id_ref;
	long _class_id_ref;
	long _seq_counter;
	String _col_value;
	String _trace_id_ref;

	@Override
	public int getSize() {
		return (getLongSize() + // trans_thread_id_ref
				getLongSize() + // thread_id_ref
				getLongSize() + // timestamp
				getIntSize() + // ticket
				getLongSize() + // thread_cpu_time
				getLongSize() + // method_id_ref
				getLongSize() + // trans_obj_id_ref
				getLongSize() + // obj_id_ref
				getLongSize() + // trans_class_id_ref
				getLongSize() + // class_id_ref
				getLongSize() + // seq_counter
				getStringSize(_col_value) +
				getStringSize(_trace_id_ref));

	}

	@Override
	public short getId() {
		return BF_METHOD_EXIT_ID;
	}
	
	@Override
	public void write(OutputStream os) throws IOException {
		super.write(os);
		writeLong(os, _trans_thread_id_ref);
		writeLong(os, _thread_id_ref);
		writeLong(os, _timestamp);
		writeInt(os, _ticket);
		writeLong(os, _thread_cpu_time);
		writeLong(os, _method_id_ref);
		writeLong(os, _trans_obj_id_ref);
		writeLong(os, _obj_id_ref);
		writeLong(os, _trans_class_id_ref);
		writeLong(os, _class_id_ref);
		writeLong(os, _seq_counter);
		writeString(os, _col_value);
		writeString(os, _trace_id_ref);

	}
	
}





class CGCStartMessage extends AbstractBinaryMessage {
	long _trans_thread_id_ref;
	long _thread_id_ref;
	long _timestamp;
	String _col_value;
	String _trace_id_ref;

	@Override
	public int getSize() {
		return (getLongSize() + // trans_thread_id_ref
				getLongSize() + // thread_id_ref
				getLongSize() + // timestamp
				getStringSize(_col_value) +
				getStringSize(_trace_id_ref));
	}

	@Override
	public short getId() {
		return BF_GC_START_ID;
	}
	
	@Override
	public void write(OutputStream os) throws IOException {
		super.write(os);
		writeLong(os, _trans_thread_id_ref);
		writeLong(os, _thread_id_ref);
		writeLong(os, _timestamp);
		writeString(os, _col_value);
		writeString(os, _trace_id_ref);		
	}
	
}



class CObjFreeMessage extends AbstractBinaryMessage {
	long _trans_thread_id_ref;
	long _thread_id_ref;
	long _timestamp;
	long _trans_obj_id_ref;
	long _obj_id_ref;
	long _obj_age;
	long _seq_counter;
	long _stack_depth;
	String _col_value;
	String _trace_id_ref;

	@Override
	public int getSize() {
		return (getLongSize() + // trans_thread_id_ref
				getLongSize() + // thread_id_ref
				getLongSize() + // timestamp
				getLongSize() + // trans_obj_id_ref
				getLongSize() + // obj_id_ref
				getLongSize() + // obj_age
				getLongSize() + // seq_counter
				getLongSize() + // stack_depth
				getStringSize(_col_value) +
				getStringSize(_trace_id_ref));
	}

	@Override
	public short getId() {
		return BF_OBJ_FREE_ID;
	}
	
	@Override
	public void write(OutputStream os) throws IOException {
		super.write(os);
		writeLong(os, _trans_thread_id_ref);
		writeLong(os, _thread_id_ref);
		writeLong(os, _timestamp);
		writeLong(os, _trans_obj_id_ref);
		writeLong(os, _obj_id_ref);
		writeLong(os, _obj_age);
		writeLong(os, _seq_counter);
		writeLong(os, _stack_depth);
		writeString(os, _col_value);
		writeString(os, _trace_id_ref);
		
	}
	
}


class CGCFinishMessage extends AbstractBinaryMessage {
	long _trans_thread_id_ref;
	long _thread_id_ref;
	long _timestamp;
	long _total_object_space;
	long _used_object_space;
	long _used_objects;
	String _col_value;
	String _trace_id_ref;
	

	@Override
	public int getSize() {
		return (getLongSize() + // _trans_thread_id_ref
				getLongSize() + // _thread_id_ref
				getLongSize() + // _timestamp
				getLongSize() + // _total_object_space
				getLongSize() + // _used_object_space
				getLongSize() + // _used_objects
				getStringSize(_col_value) +
				getStringSize(_trace_id_ref));
	}

	@Override
	public short getId() {
		return BF_GC_FINISH_ID;
	}
	
	@Override
	public void write(OutputStream os) throws IOException {
		super.write(os);
		
		writeLong(os, _trans_thread_id_ref);
		writeLong(os, _thread_id_ref);
		writeLong(os, _timestamp);
		writeLong(os, _total_object_space);
		writeLong(os, _used_object_space);
		writeLong(os, _used_objects);
	
		writeString(os, _col_value);
		writeString(os, _trace_id_ref);		
		
	}
	
}


class CRuntimeInitDoneMessage extends AbstractBinaryMessage {
	long _trans_thread_id_ref;
	long _thread_id_ref;
	long _timestamp;
	String _col_value;
	String _trace_id_ref;

	@Override
	public int getSize() {
		return (getLongSize() + // trans_thread_id_ref
				getLongSize() + // thread_id_ref
				getLongSize() + // timestamp
				getStringSize(_col_value) +
				getStringSize(_trace_id_ref));
	}

	@Override
	public short getId() {
		return BF_RUNTIME_INIT_DONE_ID;
	}
	
	@Override
	public void write(OutputStream os) throws IOException {
		super.write(os);
		writeLong(os, _trans_thread_id_ref);
		writeLong(os, _thread_id_ref);
		writeLong(os, _timestamp);
		writeString(os, _col_value);
		writeString(os, _trace_id_ref);
	}
	
}


class CRuntimeShutdownMessage extends AbstractBinaryMessage {
	long _trans_thread_id_ref;
	long _thread_id_ref;
	long _timestamp;
	String _col_value;
	String _trace_id_ref;

	@Override
	public int getSize() {
		return (getLongSize() + // _trans_thread_id_ref
				getLongSize() + // _thread_id_ref
				getLongSize() + // _timestamp
				getStringSize(_col_value) +
				getStringSize(_trace_id_ref));
	}

	@Override
	public short getId() {
		return BF_RUNTIME_SHUTDOWN_ID;
	}
	
	@Override
	public void write(OutputStream os) throws IOException {
		super.write(os);
		
		writeLong(os, _trans_thread_id_ref);
		writeLong(os, _thread_id_ref);
		writeLong(os, _timestamp);
		writeString(os, _col_value);
		writeString(os, _trace_id_ref);
		
	}
	
}


class CAGMethodEntryMessage extends AbstractBinaryMessage {
	long _thread_id_ref;
	long _method_id_ref;
	long _base_time;
	long _min_time;
	long _max_time;
	long _base_cpu_time;
	long _num_calls;

	@Override
	public int getSize() {
		return (getLongSize() + // thread_id_ref
				getLongSize() + // method_id_ref
				getLongSize() + // base_time
				getLongSize() + // min_time
				getLongSize() + // max_time
				getLongSize() + // base_cpu_time
				getLongSize()); // num_calls
	}

	@Override
	public short getId() {
		return BF_AG_METHOD_ENTRY_ID;
	}
	
	@Override
	public void write(OutputStream os) throws IOException {
		super.write(os);
		
		writeLong(os, _thread_id_ref);
		writeLong(os, _method_id_ref);
		writeLong(os, _base_time);
		writeLong(os, _min_time);
		writeLong(os, _max_time);
		writeLong(os, _base_cpu_time);
		writeLong(os, _num_calls);		
	}
	
}


class CAGMethodExitMessage extends AbstractBinaryMessage {
	long _thread_id_ref;
	long _method_id_ref;

	@Override
	public int getSize() {
		return (getLongSize() + // thread_id_ref
				getLongSize()); // method_id_ref
	}

	@Override
	public short getId() {
		return BF_AG_METHOD_EXIT_ID;
	}
	
	@Override
	public void write(OutputStream os) throws IOException {
		super.write(os);
		writeLong(os, _thread_id_ref);
		writeLong(os, _method_id_ref);
		
	}
	
}
