/*******************************************************************************
 * Copyright (c) 2007, 2014 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - Initial API and implementation
 *******************************************************************************/
package org.eclipse.tptp.platform.profile.server.wst.internal;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.hyades.trace.internal.ui.TraceConstants;
import org.eclipse.hyades.trace.ui.internal.launcher.HyadesTraceUIExtensionSupportUtil;
import org.eclipse.hyades.trace.ui.internal.launcher.IProfileLaunchConfigurationConstants;
import org.eclipse.jdt.launching.IVMInstall;
import org.eclipse.jdt.launching.IVMInstall2;
import org.eclipse.jdt.launching.IVMInstall3;
import org.eclipse.jdt.launching.VMRunnerConfiguration;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.jst.server.core.ServerProfilerDelegate;
import org.eclipse.swt.widgets.Display;
import org.eclipse.tptp.platform.iac.administrator.internal.common.AdminUtil;
import org.eclipse.tptp.platform.iac.administrator.internal.startstop.AutoStartStop;
import org.eclipse.tptp.platform.profile.server.core.internal.IProfileServerConstants;
import org.eclipse.tptp.platform.profile.server.wst.provisional.ServerProfilingUtils;
import org.eclipse.tptp.trace.ui.internal.launcher.core.DataCollectorManager;
import org.eclipse.tptp.trace.ui.internal.launcher.core.LauncherUtility;
import org.eclipse.tptp.trace.ui.provisional.launcher.IDataCollectorBaseLauncher;
import org.eclipse.tptp.trace.ui.provisional.launcher.IParameterExtractor;

public class TPTPServerProfilerDelegate extends ServerProfilerDelegate{

	private String[] vm_arg;
	private boolean usingAgentPath;
	
	protected String[] processVMArgs(String[] vmArgs, final ILaunchConfigurationWorkingCopy wc) {
		
		/*
		 * There are two dialogs possibily shown during profile on server process, 
		 * 1. launch dialog : is shown before server is started in profiling mode, this dialog is for user to pick the 
		 * 						agent type (PI or TI) and as well select analysis type.
		 * 2. attach dialog : is shown after server is started in profiling mode with a profiling agent associated, and 
		 * 						analysis type of the agent type available is available.
		 * 
		 * launch dialog is shown for user to pick the data collector and analysis type, the options are then persisted
		 * and reapplied in the attach step, attach dialog is shown when there are decision needs to be made by user. ie.
		 * multiple data collector is deteched on server host, or it will be skipped from showning, and options are set 
		 * behide the cover.
		 */
		
		Display.getDefault().syncExec(new Runnable()
		{
			public void run()
			{
					vm_arg = getVMArgsFromUser(wc);
			}
		});				
		
		if (vm_arg!=null)	{
			String[] s;
			s = new String[vmArgs.length + vm_arg.length];
			System.arraycopy(vm_arg, 0, s, 0,vm_arg.length);
			System.arraycopy(vmArgs, 0, s, vm_arg.length, vmArgs.length);
			return s;
		}
		
		return vmArgs;
	}
	
	/*
	 * remove any leading spaces in a string
	 */
	protected String trimLeadingSpaces(String str) {
		String result = str;
		while (result.startsWith(" ")) {
			result = result.substring(1, result.length());
		}
		return result;
	}
	
	/**
	 * Prompt the user for profiling arguments and returns an array of JVM arguments corresponding
	 * to the selected mode.
	 * 
	 * Upon return, the 'usingAgentPath' member var is set to true or false to indicate whether or
	 * not the -agentpath argument is being used.
	 * 
	 * @param wc 
	 * @return Array of arguments to append to JVM launch, or null if user cancelled the operation.
	 * 	
	 */
	protected String[] getVMArgsFromUser(ILaunchConfigurationWorkingCopy wc) {

		// Assume we aren't using this until we actually insert the argument.
		usingAgentPath = false;
		
		try {
			// This attribute is used to auto launch a profile through GBP. 
			// Here we clear (set to false) the attribute for this launch.
			wc.setAttribute(TraceConstants.LAUNCH_CONFIG_ALLOW_LAUNCH, false);
			wc.doSave();
		} catch (CoreException e1) { /* bury the exception */}
		
		// Launch the profiling option wizard
		ProfileOnServerOptionWizard wizard = new ProfileOnServerOptionWizard(wc);
        WizardDialog dialog = new WizardDialog(ServerPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(), wizard);
        dialog.create();
        dialog.open();
        
    	int wizardReturnCode = dialog.getReturnCode();
    	
    	try {
			if(wc.getAttribute(TraceConstants.LAUNCH_CONFIG_ALLOW_LAUNCH, false)) {
				wizardReturnCode = 0; // 0 means not cancelled, 1 is cancelled

				// Reset the attribute back to false
				wc.setAttribute(TraceConstants.LAUNCH_CONFIG_ALLOW_LAUNCH, false);
				wc.doSave();
				
			}
		} catch (CoreException e1) {}
    	
    	
    	
    	wc.setAttribute(IProfileServerConstants.SERVER_DIALOG_STATUS, wizardReturnCode);
    	if (wizardReturnCode == Dialog.CANCEL) {
            return null;
        }
    	
		try {
			String collectorOptions = wc.getAttribute(IProfileLaunchConfigurationConstants.ATTR_PROFILING_COLLECTOR_AND_ANALYSIS,"");
			String[] collectors = collectorOptions.split(";");
			ArrayList list = new ArrayList();
			for (int i=0;i<collectors.length;i++) {
				int idx = collectors[i].indexOf(":");
				if (idx>0) {
					String collectorID = collectors[i].substring(0, idx);
					
					IDataCollectorBaseLauncher delegate = DataCollectorManager.getInstance().getDataCollectorLaunchDelegate(collectorID, wc.getOriginal());
					if (delegate instanceof IParameterExtractor) {
						List paramList = ((IParameterExtractor)delegate).getProcessParameters(wc.getOriginal()).getExecutableParameters();
						if (paramList!=null)
							list.addAll(paramList);
					}	
				}
			}
			String[] vmArgStr = new String[list.size()];
			list.toArray(vmArgStr);

			// If jvm_arch is specified, use -agentpath: to point at the correct profiling libs
			String jvmtiPATH = null;
			String jvm_arch = wc.getAttribute(IProfileServerConstants.SERVER_JVM_ARCH, (String)null);
			if( null != jvm_arch ) {
				jvmtiPATH = AdminUtil.getJvmtiNativePath(jvm_arch);
			}

			//replace server=controlled with server=enabled for TI vm arguments
			Pattern pattern = Pattern.compile("server=controlled");
			for (int i=0;i<vmArgStr.length;i++) {
				
				String arg = vmArgStr[i];
				
		        Matcher m = pattern.matcher(arg); // get a matcher object
		        arg = m.replaceAll("server=enabled");
		        
				boolean quoted = false;
				if( arg.startsWith("\"") )
					quoted = true;
				
				if ( null != jvmtiPATH && (arg.startsWith("\"-agentlib:") || arg.startsWith("-agentlib:")) ) {
					arg = arg.replace("-agentlib:", "-agentpath:"+jvmtiPATH+File.separatorChar);
					arg = arg.replace("JPIBootLoader", IProfileLaunchConfigurationConstants.LIB_PREFIX + "JPIBootLoader" + IProfileLaunchConfigurationConstants.LIB_SUFFIX);
					
					usingAgentPath = true;
				}
				
				// Remove quotes because the latter parts of the WTP launch process don't want them 
				if( quoted )
					arg = arg.substring(1, arg.length()-1);
		        vmArgStr[i]= arg.trim(); //bug 195367
			}
	        
			return vmArgStr;
		} catch (CoreException e) {
			e.printStackTrace();
		}
		
		return null;
	}
	
	public void startAC() {
		if (!AutoStartStop.isIACRunning()) {
			AutoStartStop.startIAC();
		}
	}
	
	public void process(ILaunch launch, IVMInstall vmInstall,VMRunnerConfiguration vmConfig, IProgressMonitor monitor)	throws CoreException {
		
		// (Attempt to) Determine the server JVM version and architecture ..
		String jvm_version=IProfileServerConstants.JVM_142; //assume JVM version to 1.4.2 by default.
		IVMInstall2 vmInstall2 = (IVMInstall2)vmInstall;
		if (vmInstall2 != null) {
			jvm_version = vmInstall2.getJavaVersion();
		}
		
		String jvm_arch = System.getProperty("os.arch").toLowerCase();
		if( vmInstall instanceof IVMInstall3 ) {
			IVMInstall3 vmInstall3 = (IVMInstall3)vmInstall;
			String osArch = (String)vmInstall3.evaluateSystemProperties( new String[] { "os.arch" }, new SubProgressMonitor(monitor, 1) ).get("os.arch");
			if( null != osArch )
				jvm_arch = osArch;
		}
		
		// .. and store in launch config
		ILaunchConfigurationWorkingCopy wc = launch.getLaunchConfiguration().getWorkingCopy();
				
		wc.setAttribute(IProfileServerConstants.SERVER_JVM_VERSION, jvm_version);
		wc.setAttribute(IProfileServerConstants.SERVER_JVM_ARCH, jvm_arch);
		wc.doSave();
		
		startAC();
		
		//insert profile mode variables
		String[] vmArgs = vmConfig.getVMArguments();
		vmArgs = processVMArgs(vmArgs, wc);

		if( jvm_version.indexOf("1.7") >= 0 && IProfileLaunchConfigurationConstants.NOVERIFY_JAVA7 ) {
			
			String[] newArgs = new String[ vmArgs.length+1 ];
			newArgs[0] = "-Xverify:none";
			System.arraycopy(vmArgs, 0, newArgs, 1, vmArgs.length);
			vmArgs = newArgs;
			
		}
		// Check for the health center arguments
		int matches = 0;
		for(int i=0; i<vmArgs.length; i++) {
			String str = vmArgs[i].toLowerCase();
			
			if(str.contains("-xhealthcenter:transport=jrmp")) {
				matches++;
			}

			if(str.contains("-dtptphcimpl=tptp")) {
				matches++;
			}
		}
		
		// If the health center arguments are present, set a variable to true in the launch configuration
		if(matches >= 2) {
			wc.setAttribute(IProfileLaunchConfigurationConstants.ATTR_PROFILING_GENERIC_AGENT_ATTACH, true);
		} else {
			// ... otherwise, set the variable to false
			wc.setAttribute(IProfileLaunchConfigurationConstants.ATTR_PROFILING_GENERIC_AGENT_ATTACH, false);
		} 
		
		// If the user cancelled the profile, then throw a CoreException to exit
		if (wc.getAttribute(IProfileServerConstants.SERVER_DIALOG_STATUS, Dialog.CANCEL)==Dialog.CANCEL) {
			throw new CoreException(new Status(IStatus.WARNING,ServerPlugin.getPluginId(),"Profile request is cancelled by user"));
		}

		vmConfig.setVMArguments(vmArgs);
		
		// We only do the env vars if we are not using the -agentpath style JVM argument. This is 
		// set by getJVMArgsFromUser
		if( !usingAgentPath ) {
			String[] envcfg = vmConfig.getEnvironment();
			String[] env = ServerProfilingUtils.getEnvironmentVariables();
			envcfg = ServerProfilingUtils.map2String(ServerProfilingUtils.processEnvironmentVariables(ServerProfilingUtils.String2map(envcfg), ServerProfilingUtils.String2map(env)));		
			vmConfig.setEnvironment(envcfg);
		}
		
		// Calculate filters
		HyadesTraceUIExtensionSupportUtil.applyExtensionFilterProcessing(wc, LauncherUtility.getActiveWorkingSets());
		
		wc.doSave();
		//pass to extension point for extra variable setup
		postProcess(launch, vmInstall, vmConfig, monitor);
	}

	protected void postProcess(ILaunch launch, IVMInstall vmInstall, VMRunnerConfiguration vmConfig, IProgressMonitor monitor) {
		ServerProfileExtender[] extenders = ServerPlugin.getServerProfileExtenders();
		for (int i=0;i<extenders.length;i++) {
			extenders[i].process(launch, vmInstall, vmConfig, monitor);
		}
	}
}
